"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.inputBodyPostSchema = void 0;
const inputBodySchemaJson = require("../schema/input-body-schema.json");
const helper_1 = require("../../../helper");
function inputBodyPostSchema(entity, fieldsArray, relationsField, schemaName, arrayPropsConfig) {
    const json = JSON.parse(JSON.stringify(inputBodySchemaJson));
    json.$id = `${json.$id}/${schemaName}`;
    json.properties.data.properties.type.enum.push((0, helper_1.camelToKebab)((0, helper_1.getEntityName)(entity)));
    const baseRelDataType = {
        type: 'object',
        properties: {
            data: {
                type: 'object',
                properties: {
                    type: {
                        type: 'string',
                    },
                    id: {
                        type: 'string',
                        pattern: '^\\d+$',
                        description: 'Use string should be as number string',
                    },
                },
                required: ['type', 'id'],
            },
        },
        required: ['data'],
    };
    const attributes = fieldsArray
        .filter((item) => item !== 'id')
        .reduce((acum, item) => {
        const typeMetadata = Reflect.getMetadata('design:type', entity['prototype'], item);
        let dataType = {};
        switch (typeMetadata) {
            case Array:
                dataType = {
                    type: 'array',
                    items: {
                        type: 'string',
                    },
                };
                break;
            case Date:
                dataType = {
                    type: 'string',
                };
                break;
            case Number:
                dataType = {
                    type: 'integer',
                };
                break;
            case Boolean:
                dataType = {
                    type: 'boolean',
                };
                break;
            case Object:
                dataType = {
                    type: 'object',
                };
                break;
            default:
                dataType = {
                    type: 'string',
                };
        }
        acum[item] = arrayPropsConfig.arrayProps[item]
            ? {
                type: 'array',
                items: {
                    type: 'string',
                },
            }
            : dataType;
        return acum;
    }, {});
    json.properties.data.properties.attributes.properties = Object.assign(Object.assign({}, json.properties.data.properties.attributes.properties), attributes);
    const uuidRelations = arrayPropsConfig.relationUuids || {};
    const relationships = Object.keys(relationsField).reduce((acum, item) => {
        var _a, _b, _c;
        const relDataType = {
            [item]: Object.assign(Object.assign({}, baseRelDataType), { properties: Object.assign(Object.assign({}, baseRelDataType.properties), { data: Object.assign(Object.assign({}, baseRelDataType.properties.data), { properties: Object.assign(Object.assign({}, baseRelDataType.properties.data.properties), { id: Object.assign(Object.assign(Object.assign(Object.assign({}, baseRelDataType.properties.data.properties.id), { pattern: ((_a = uuidRelations[item]) === null || _a === void 0 ? void 0 : _a.id)
                                    ? '^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$'
                                    : '^\\d+$' }), (((_b = uuidRelations[item]) === null || _b === void 0 ? void 0 : _b.id)
                                ? {
                                    minLength: 36,
                                    maxLength: 36,
                                }
                                : {})), { description: ((_c = uuidRelations[item]) === null || _c === void 0 ? void 0 : _c.id)
                                    ? 'Use string should be as uuid string'
                                    : 'Use string should be as number string' }) }) }) }) }),
        };
        const resultSchema = Object.assign(Object.assign({}, relDataType[item].properties.data), { properties: Object.assign(Object.assign({}, relDataType[item].properties.data.properties), { type: Object.assign(Object.assign({}, relDataType[item].properties.data.properties.type), (arrayPropsConfig.relationType[item]
                    ? {
                        enum: [
                            (0, helper_1.camelToKebab)((0, helper_1.getEntityName)(arrayPropsConfig.relationType[item])),
                        ],
                    }
                    : {})) }) });
        acum[item] = Object.assign(Object.assign({}, relDataType[item]), { properties: Object.assign(Object.assign({}, relDataType[item].properties), { data: Reflect.getMetadata('design:type', entity['prototype'], item) ===
                    Array
                    ? {
                        type: 'array',
                        items: resultSchema,
                        minItems: 1,
                    }
                    : resultSchema }) });
        return acum;
    }, {});
    json.properties.data.properties.relationships.properties = Object.assign(Object.assign({}, json.properties.data.properties.relationships.properties), relationships);
    json.properties.data.properties.relationships = Object.assign({}, json.properties.data.properties.relationships);
    return json;
}
exports.inputBodyPostSchema = inputBodyPostSchema;
//# sourceMappingURL=input-body-post-schema.js.map