"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BodyRelationshipPatchPipe = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const ajv_1 = require("ajv");
const helper_1 = require("../../../helper");
let BodyRelationshipPatchPipe = class BodyRelationshipPatchPipe {
    constructor(ajvCall) {
        this.ajvCall = ajvCall;
        this.validateFunction = this.ajvCall.getSchema('body-relationship-patch-schema');
    }
    transform(value) {
        const validate = this.validateFunction(value);
        const errorResult = [];
        if (!validate) {
            for (const error of this.validateFunction.errors) {
                const errorMsg = [];
                errorMsg.push((0, helper_1.upperFirstLetter)(error.message));
                switch (error.keyword) {
                    case 'enum':
                        errorMsg.push(`Allowed values are: "${error.params.allowedValues.join(',')}"`);
                        break;
                    case 'additionalProperties':
                        errorMsg.push(`Additional Property is: "${error.params.additionalProperty}"`);
                        break;
                    case 'oneOf':
                        if (error.schemaPath === '#/$defs/dataSchema/oneOf') {
                            errorMsg[errorMsg.length - 1] =
                                'Must match exactly one schema: "null" or "object"';
                        }
                        else {
                            errorMsg[errorMsg.length - 1] =
                                'Must match exactly one schema: "object" or "array"';
                        }
                        break;
                }
                errorResult.push({
                    source: {
                        parameter: error.instancePath,
                    },
                    detail: errorMsg.join('. '),
                });
            }
        }
        if (errorResult.length > 0) {
            throw new common_1.BadRequestException(errorResult);
        }
        return value.data;
    }
};
BodyRelationshipPatchPipe = tslib_1.__decorate([
    (0, common_1.Injectable)(),
    tslib_1.__param(0, (0, common_1.Inject)(ajv_1.default)),
    tslib_1.__metadata("design:paramtypes", [ajv_1.default])
], BodyRelationshipPatchPipe);
exports.BodyRelationshipPatchPipe = BodyRelationshipPatchPipe;
//# sourceMappingURL=body-relationship-patch.pipe.js.map