"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAll = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const helper_1 = require("../../../../../helper");
function getAll(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const startTime = Date.now();
        const { filter, include, sort, page, fields } = options.query;
        if (this.config.requiredSelectField && fields === null) {
            throw new common_1.BadRequestException([
                {
                    source: { parameter: '/fields' },
                    detail: 'Fields params in query is required',
                },
            ]);
        }
        const preparedResourceName = (0, helper_1.snakeToCamel)(this.repository.metadata.name);
        const builder = this.repository.createQueryBuilder(preparedResourceName);
        const resultBuilderQuery = this.repository.createQueryBuilder(preparedResourceName);
        if (!this.UtilsMethode.relationNameMapByEntity.has(preparedResourceName)) {
            this.UtilsMethode.relationNameMapByEntity.set(preparedResourceName, new Set(this.repository.metadata.relations.map((item) => item.propertyName)));
        }
        const fieldsSelect = (include || []).reduce((acum, i) => {
            acum.add(i);
            return acum;
        }, new Set());
        fieldsSelect.add(preparedResourceName);
        if (fields) {
            fieldsSelect.clear();
            for (const rel of include || []) {
                const propsName = this.repository.metadata.relations.find((relations) => relations.propertyName === rel).entityMetadata.primaryColumns[0].propertyName;
                fieldsSelect.add(`${rel}.${propsName}`);
            }
            const { target } = fields, other = tslib_1.__rest(fields, ["target"]);
            const targetArray = [...target, this.repository.metadata.primaryColumns[0].propertyName] ||
                [];
            for (const fieldTarget of targetArray) {
                fieldsSelect.add(`${preparedResourceName}.${fieldTarget}`);
            }
            Object.keys(other || {}).forEach((relation) => {
                (other[relation] || []).forEach((i) => {
                    fieldsSelect.add(`${relation}.${i}`);
                });
            });
        }
        const { target, relation } = filter;
        const expressionObjectForRelation = relation
            ? this.UtilsMethode.applyQueryFilterRelation(builder, relation, this.repository.metadata)
            : [];
        const expressionObject = [
            ...(target
                ? this.UtilsMethode.applyQueryFiltersTarget(builder, target, this.repository.metadata)
                : []),
            ...expressionObjectForRelation,
        ];
        const expressionObjectLength = expressionObject.length;
        const includeLength = include ? include.length : 0;
        const joinForCommonQuery = {};
        for (let i = 0; i < expressionObjectLength; i++) {
            const { expression, params, selectInclude } = expressionObject[i];
            if (selectInclude) {
                joinForCommonQuery[`${preparedResourceName}.${selectInclude}`] =
                    selectInclude;
            }
            builder[i === 0 ? 'where' : 'andWhere'](expression);
            if (params) {
                builder.setParameters(params ? { [params.name]: params.val } : {});
            }
        }
        for (let i = 0; i < includeLength; i++) {
            const relationName = include[i];
            resultBuilderQuery.leftJoin(`${preparedResourceName}.${relationName}`, relationName);
        }
        const subQueryIdAlias = 'subQueryId';
        const primaryColumn = this.repository.metadata.primaryColumns[0].databaseName;
        builder.select(`${preparedResourceName}.${primaryColumn}`, subQueryIdAlias);
        if (sort) {
            const { target } = sort, otherSort = tslib_1.__rest(sort, ["target"]);
            const targetOrder = Object.entries(target || {}).reduce((acum, [key, val]) => {
                acum[`${preparedResourceName}.${key}`] = val;
                return acum;
            }, {});
            const relationOrder = Object.keys(otherSort || {}).reduce((acum, relation) => {
                return Object.entries((otherSort || {})[relation]).reduce((a, [key, val]) => {
                    a[`${relation}.${key}`] = val;
                    return a;
                }, acum);
            }, {});
            for (const relItem in relationOrder) {
                const [rel, field] = relItem.split('.');
                joinForCommonQuery[`${preparedResourceName}.${rel}`] = rel;
            }
            builder.orderBy(Object.assign(Object.assign({}, targetOrder), relationOrder));
        }
        else {
            builder.orderBy({
                [`${preparedResourceName}.${primaryColumn}`]: 'ASC',
            });
        }
        for (const join in joinForCommonQuery) {
            builder.leftJoin(join, joinForCommonQuery[join]);
        }
        const prepareParams = Date.now() - startTime;
        const count = yield builder.getCount();
        const skip = (page.number - 1) * page.size;
        builder.offset(skip).limit(page.size);
        const countAlias = 'countResult';
        const resultIds = yield this.repository
            .createQueryBuilder(countAlias)
            .select(`${countAlias}.${primaryColumn}`)
            .innerJoin(`(${builder.getQuery()})`, 'subQuery', `"subQuery"."${subQueryIdAlias}" = ${countAlias}.${primaryColumn}`)
            .setParameters(builder.getParameters())
            .getRawMany();
        const resultBuilder = resultBuilderQuery
            .select([...fieldsSelect]);
        const ids = resultIds.map((i) => i[`${countAlias}_${primaryColumn}`]);
        if (ids.length > 0) {
            resultBuilder.whereInIds(resultIds.map((i) => i[`${countAlias}_${primaryColumn}`]));
        }
        for (let i = 0; i < expressionObjectForRelation.length; i++) {
            const { expression, params, selectInclude } = expressionObjectForRelation[i];
            if (selectInclude && !include.includes(selectInclude)) {
                resultBuilder.leftJoin(`${preparedResourceName}.${selectInclude}`, selectInclude);
            }
            if (i === 0 && ids.length === 0) {
                resultBuilder.where(expression);
            }
            else {
                resultBuilder.andWhere(expression);
            }
            resultBuilder.setParameters(params ? { [params.name]: params.val } : {});
        }
        const result = yield resultBuilder.getRawMany();
        const callQuery = Date.now() - startTime;
        const entityArray = this.transform.transformRawData(result);
        const data = entityArray.map((i) => this.transform.transformData(i, include));
        const included = this.transform.transformInclude(entityArray);
        const transform = Date.now() - startTime;
        const debug = {
            prepareParams,
            callQuery: callQuery - prepareParams,
            transform: transform - callQuery,
        };
        return {
            meta: Object.assign({ pageNumber: page.number, totalItems: count, pageSize: page.size }, (this.config.debug ? { debug } : {})),
            data,
            included,
        };
    });
}
exports.getAll = getAll;
//# sourceMappingURL=get-all.js.map